from bs4 import BeautifulSoup

from um_ro_calendar.entities import CheckboxOption, FormFeatures, TermOption


def _tag_for_html(html):
    return BeautifulSoup(html, features="html.parser").find()


def test_checkbox_option_from_beautifulsoup_tag():
    tag = _tag_for_html('<input type="checkbox" name="name" value="value">')
    checkbox_option = CheckboxOption.from_tag(tag)
    assert checkbox_option.name == "name"
    assert checkbox_option.value == "value"


example_features = FormFeatures(
    "term_keyword",
    [TermOption("1", "Fall 2022"), TermOption("2", "Winter 2023")],
    CheckboxOption("accal", "1"),
    CheckboxOption("reg", "1"),
)


def test_term_option_search_success():
    term_option = example_features.term_option_for_name("Fall 2022")
    assert term_option is not None
    assert term_option.id == "1"


def test_term_option_search_failure():
    term_option = example_features.term_option_for_name("")
    assert term_option is None


def test_get_url_default():
    url = example_features.get_url("Fall 2022", "http://localhost/cal")
    assert "term_keyword=1" in url
    assert "accal=1" in url
    assert "reg=1" not in url


def test_get_url_with_reg_deadlines_only():
    url = example_features.get_url(
        "Fall 2022", "", academic_calendar=False, registration_deadlines=True
    )
    assert "term_keyword=1" in url
    assert "accal=1" not in url
    assert "reg=1" in url
