from typing import cast

from bs4 import BeautifulSoup, Tag

from .entities import CheckboxOption, FormFeatures, TermOption
from .utils import fetch_url


class FormParser:
    url: str

    def __init__(self, url: str = "https://ro.umich.edu/calendars"):
        self.url = url

    def process_output(self, html: str) -> FormFeatures:
        soup = BeautifulSoup(html, features="html.parser")
        academic_term_label = cast(
            Tag, soup.find(name="label", string="Academic Term*")
        )
        academic_calendar_label = cast(
            Tag, soup.find(name="label", string="Academic Calendar")
        )
        registration_deadlines_label = cast(
            Tag, soup.find(name="label", string="Registration Deadlines")
        )

        if not all(
            [academic_term_label, academic_calendar_label, registration_deadlines_label]
        ):
            raise ValueError("No valid form found")

        academic_term_select = cast(Tag, soup.find(id=academic_term_label.attrs["for"]))
        academic_calendar_checkbox = cast(
            Tag, soup.find(id=academic_calendar_label.attrs["for"])
        )
        registration_deadlines_checkbox = cast(
            Tag, soup.find(id=registration_deadlines_label.attrs["for"])
        )

        term_options = [
            TermOption(t.attrs["value"], t.string)
            for t in academic_term_select.find_all("option")
        ]

        return FormFeatures(
            academic_term_select.attrs["name"],
            term_options,
            CheckboxOption.from_tag(academic_calendar_checkbox),
            CheckboxOption.from_tag(registration_deadlines_checkbox),
        )

    def fetch_html(self) -> str:
        return fetch_url(self.url)

    def process(self) -> FormFeatures:
        html = self.fetch_html()
        return self.process_output(html)
