from datetime import timedelta
from typing import Iterable
from urllib.request import Request, urlopen

from .entities import Event


def fetch_url(url: str) -> str:
    """This request spoofing is done to get around Cloudflare's scraping block"""

    request = Request(url)
    request.add_header(
        "User-Agent",
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:106.0)"
        "Gecko/20100101 Firefox/106.0",
    )
    request.add_header(
        "Accept",
        "text/html,application/xhtml+xml,application/xml;q=0.9,"
        "image/avif,image/webp,*/*;q=0.8",
    )
    request.add_header("Accept-Language", "en-US,en;q=0.5")
    return urlopen(request).read().decode("utf-8")


def fix_spring_break_spans(events: Iterable[Event]) -> Iterable[Event]:
    vacation_begins = None
    classes_resume = None

    for event in events:
        if event.name.startswith("Vacation begins at noon"):
            vacation_begins = event
        elif event.name.startswith("Classes resume 8"):
            classes_resume = event

        if vacation_begins:
            if classes_resume:
                yield Event(
                    "Spring Break",
                    vacation_begins.start_date,
                    classes_resume.end_date - timedelta(days=1),
                )
                vacation_begins = None
                classes_resume = None
        else:
            yield event
